@@ -1,13 +1,13 @@
 # -*- test-case-name: wokkel.test.test_shim -*-
 #
-# Copyright (c) 2003-2008 Ralph Meijer
+# Copyright (c) Ralph Meijer.
 # See LICENSE for details.
 
 """
 XMPP Stanza Headers and Internet Metadata.
 
 This protocol is specified in
-U{XEP-0131<http://www.xmpp.org/extensions/xep-0131.html>}.
+U{XEP-0131<http://xmpp.org/extensions/xep-0131.html>}.
 """
 
 from twisted.words.xish import domish
