@@ -7,21 +7,27 @@
 XMPP IM protocol support.
 
 This module provides generic implementations for the protocols defined in
-U{RFC 3921<http://xmpp.org/rfcs/rfc3921.html>} (XMPP IM).
-
-All of it should eventually move to Twisted.
+U{RFC 6121<http://www.xmpp.org/rfcs/rfc6121.html>} (XMPP IM).
 """
 
+import warnings
+
+from twisted.internet import defer
+from twisted.words.protocols.jabber import error
 from twisted.words.protocols.jabber.jid import JID
 from twisted.words.xish import domish
 
-from wokkel.compat import IQ
-from wokkel.generic import ErrorStanza, Stanza
+from wokkel.generic import ErrorStanza, Stanza, Request
+from wokkel.subprotocols import IQHandlerMixin
 from wokkel.subprotocols import XMPPHandler
 
 NS_XML = 'http://www.w3.org/XML/1998/namespace'
 NS_ROSTER = 'jabber:iq:roster'
 
+XPATH_ROSTER_SET = "/iq[@type='set']/query[@xmlns='%s']" % NS_ROSTER
+
+
+
 class Presence(domish.Element):
     def __init__(self, to=None, type=None):
         domish.Element.__init__(self, (None, "presence"))
@@ -605,9 +611,9 @@ class RosterItem(object):
 
     This represents one contact from an XMPP contact list known as roster.
 
-    @ivar jid: The JID of the contact.
-    @type jid: L{JID}
-    @ivar name: The optional associated nickname for this contact.
+    @ivar entity: The JID of the contact.
+    @type entity: L{JID}
+    @ivar name: The associated nickname for this contact.
     @type name: C{unicode}
     @ivar subscriptionTo: Subscription state to contact's presence. If C{True},
                           the roster owner is subscribed to the presence
@@ -616,116 +622,382 @@ class RosterItem(object):
     @ivar subscriptionFrom: Contact's subscription state. If C{True}, the
                             contact is subscribed to the presence information
                             of the roster owner.
-    @type subscriptionTo: C{bool}
-    @ivar ask: Whether subscription is pending.
-    @type ask: C{bool}
+    @type subscriptionFrom: C{bool}
+    @ivar pendingOut: Whether the subscription request to this contact is
+        pending.
+    @type pendingOut: C{bool}
     @ivar groups: Set of groups this contact is categorized in. Groups are
                   represented by an opaque identifier of type C{unicode}.
     @type groups: C{set}
+    @ivar approved: Signals pre-approved subscription.
+    @type approved: C{bool}
+    @ivar remove: Signals roster item removal.
+    @type remove: C{bool}
     """
 
-    def __init__(self, jid):
-        self.jid = jid
-        self.name = None
-        self.subscriptionTo = False
-        self.subscriptionFrom = False
-        self.ask = None
-        self.groups = set()
+    __subscriptionStates = {(False, False): None,
+                            (True, False): 'to',
+                            (False, True): 'from',
+                            (True, True): 'both'}
+
+    def __init__(self, entity, subscriptionTo=False, subscriptionFrom=False,
+                       name=u'', groups=None):
+        self.entity = entity
+        self.subscriptionTo = subscriptionTo
+        self.subscriptionFrom = subscriptionFrom
+        self.name = name
+        self.groups = groups or set()
+
+        self.pendingOut = False
+        self.approved = False
+        self.remove = False
 
 
-class RosterClientProtocol(XMPPHandler):
-    """
-    Client side XMPP roster protocol.
-    """
+    def __getJID(self):
+        warnings.warn(
+            "wokkel.xmppim.RosterItem.jid was deprecated in Wokkel 0.7.1; "
+            "please use RosterItem.entity instead.",
+            DeprecationWarning)
+        return self.entity
 
-    def connectionInitialized(self):
-        ROSTER_SET = "/iq[@type='set']/query[@xmlns='%s']" % NS_ROSTER
-        self.xmlstream.addObserver(ROSTER_SET, self._onRosterSet)
 
-    def _parseRosterItem(self, element):
-        jid = JID(element['jid'])
-        item = RosterItem(jid)
-        item.name = element.getAttribute('name')
+    def __setJID(self, value):
+        warnings.warn(
+            "wokkel.xmppim.RosterItem.jid was deprecated in Wokkel 0.7.1; "
+            "please use RosterItem.entity instead.",
+            DeprecationWarning)
+        self.entity = value
+
+
+    jid = property(__getJID, __setJID,
+                   doc="JID of the contact. "
+                       "Deprecated in Wokkel 0.7.1; "
+                       "please use C{entity} instead.")
+
+
+    def __getAsk(self):
+        warnings.warn(
+            "wokkel.xmppim.RosterItem.ask was deprecated in Wokkel 0.7.1; "
+            "please use RosterItem.pendingOut instead.",
+            DeprecationWarning)
+        return self.pendingOut
+
+
+    def __setAsk(self, value):
+        warnings.warn(
+            "wokkel.xmppim.RosterItem.ask was deprecated in Wokkel 0.7.1; "
+            "please use RosterItem.pendingOut instead.",
+            DeprecationWarning)
+        self.pendingOut = value
+
+
+    ask = property(__getAsk, __setAsk,
+                   doc="Pending out subscription. "
+                       "Deprecated in Wokkel 0.7.1; "
+                       "please use C{pendingOut} instead.")
+
+
+    def toElement(self, rosterSet=False):
+        """
+        Render to a DOM representation.
+
+        If C{rosterSet} is set, some attributes, that may not be sent
+        as a roster set, will not be rendered.
+
+        @type rosterSet: C{boolean}.
+        """
+        element = domish.Element((NS_ROSTER, 'item'))
+        element['jid'] = self.entity.full()
+
+        if self.remove:
+            subscription = 'remove'
+        else:
+            if self.name:
+                element['name'] = self.name
+
+            if self.groups:
+                for group in self.groups:
+                    element.addElement('group', content=group)
+
+            if rosterSet:
+                subscription = None
+            else:
+                subscription = self.__subscriptionStates[self.subscriptionTo,
+                                                         self.subscriptionFrom]
+
+                if self.pendingOut:
+                    element['ask'] = u'subscribe'
+
+                if self.approved:
+                    element['approved'] = u'true'
+
+        if subscription:
+            element['subscription'] = subscription
+
+        return element
+
+
+    @classmethod
+    def fromElement(Class, element):
+        entity = JID(element['jid'])
+        item = Class(entity)
         subscription = element.getAttribute('subscription')
-        item.subscriptionTo = subscription in ('to', 'both')
-        item.subscriptionFrom = subscription in ('from', 'both')
-        item.ask = element.getAttribute('ask') == 'subscribe'
-        for subElement in domish.generateElementsQNamed(element.children,
-                                                        'group', NS_ROSTER):
-            item.groups.add(unicode(subElement))
-
+        if subscription == 'remove':
+            item.remove = True
+        else:
+            item.name = element.getAttribute('name', u'')
+            item.subscriptionTo = subscription in ('to', 'both')
+            item.subscriptionFrom = subscription in ('from', 'both')
+            item.pendingOut = element.getAttribute('ask') == 'subscribe'
+            item.approved = element.getAttribute('approved') in ('true', '1')
+            for subElement in domish.generateElementsQNamed(element.children,
+                                                            'group', NS_ROSTER):
+                item.groups.add(unicode(subElement))
         return item
 
-    def getRoster(self):
+
+
+class RosterRequest(Request):
+    """
+    Roster request.
+
+    @ivar item: Roster item to be set or pushed.
+    @type item: L{RosterItem}.
+
+    @ivar version: Roster version identifier for roster pushes and
+        retrieving the roster as a delta from a known cached version. This
+        should only be set if the recipient is known to support roster
+        versioning.
+    @type version: C{unicode}
+
+    @ivar rosterSet: If set, this is a roster set request. This flag is used
+        to make sure some attributes of the roster item are not rendered by
+        L{toElement}.
+    @type roster: C{boolean}
+    """
+    item = None
+    version = None
+    rosterSet = False
+
+    def parseRequest(self, element):
+        self.version = element.getAttribute('ver')
+
+        for child in element.elements(NS_ROSTER, 'item'):
+            self.item = RosterItem.fromElement(child)
+            break
+
+
+    def toElement(self):
+        element = Request.toElement(self)
+        query = element.addElement((NS_ROSTER, 'query'))
+        if self.version is not None:
+            query['ver'] = self.version
+        if self.item:
+            query.addChild(self.item.toElement(rosterSet=self.rosterSet))
+        return element
+
+
+
+class RosterPushIgnored(Exception):
+    """
+    Raised when this entity doesn't want to accept/trust a roster push.
+
+    To avert presence leaks, a handler can raise L{RosterPushIgnored} when
+    not accepting a roster push (directly or via Deferred). This will
+    result in a C{'service-unavailable'} error being sent in return.
+    """
+
+
+
+class Roster(dict):
+    """
+    In-memory roster container.
+
+    This provides a roster as a mapping from L{JID} to L{RosterItem}. If
+    roster versioning is used, the C{version} attribute holds the version
+    identifier for this version of the roster.
+
+    @ivar version: Roster version identifier.
+    @type version: C{unicode}.
+    """
+
+    version = None
+
+
+
+class RosterClientProtocol(XMPPHandler, IQHandlerMixin):
+    """
+    Client side XMPP roster protocol.
+
+    The roster can be retrieved using L{getRoster}. Subsequent changes to the
+    roster will be pushed, resulting in calls to L{setReceived} or
+    L{removeReceived}. These methods should be overridden to handle the
+    roster pushes.
+
+    RFC 6121 specifically allows entities other than a user's server to
+    hold a roster for that user. However, how a client should deal with
+    that is currently not yet specfied.
+
+    By default roster pushes from other source. I.e. when C{request.sender}
+    is set but the sender's bare JID is different from the user's bare JID.
+    Set L{allowAnySender} to allow roster pushes from any sender. To
+    avert presence leaks, L{RosterPushIgnored} should then be raised for
+    pushes from untrusted senders.
+
+    If roster versioning is supported by the server, the roster and
+    subsequent pushes are annotated with a version identifier. This can be
+    used to cache the roster on the client side. Upon reconnect, the client
+    can request the roster with the version identifier of the cached version.
+    The server may then choose to only send roster pushes for the changes
+    since that version, instead of a complete roster.
+
+    @cvar allowAnySender: Flag to allow roster pushes from any sender.
+        C{False} by default.
+    @type allowAnySender: C{boolean}
+    """
+
+    allowAnySender = False
+    iqHandlers = {XPATH_ROSTER_SET: "_onRosterSet"}
+
+
+    def connectionInitialized(self):
+        self.xmlstream.addObserver(XPATH_ROSTER_SET, self.handleRequest)
+
+
+    def getRoster(self, version=None):
         """
         Retrieve contact list.
 
+        The returned deferred fires with the result of the roster request as
+        L{Roster}, a mapping from contact JID to L{RosterItem}.
+
+        If roster versioning is supported, the recipient responds with either
+        a the complete roster or with an empty result. In case of complete
+        roster, the L{Roster} is annotated with a C{version} attribute that
+        holds the version identifier for this version of the roster. This
+        identifier should be used for caching.
+
+        If the recipient responds with an empty result, the returned deferred
+        fires with C{None}. This indicates that any roster modifications
+        since C{version} will be sent as roster pushes.
+
+        Note that the empty result (C{None}) is different from an empty
+        roster (L{Roster} with no items).
+
+        @param version: Optional version identifier of the last cashed
+            version of the roster. This shall only be set if the recipient is
+            known to support roster versioning. If there is no (valid) cached
+            version of the roster, but roster versioning is desired,
+            C{version} should be set to the empty string (C{u''}).
+        @type version: C{unicode}
+
         @return: Roster as a mapping from L{JID} to L{RosterItem}.
         @rtype: L{twisted.internet.defer.Deferred}
         """
 
         def processRoster(result):
-            roster = {}
-            for element in domish.generateElementsQNamed(result.query.children,
-                                                         'item', NS_ROSTER):
-                item = self._parseRosterItem(element)
-                roster[item.jid.userhost()] = item
+            if result.query is not None:
+                roster = Roster()
+                roster.version = result.query.getAttribute('ver')
+                for element in result.query.elements(NS_ROSTER, 'item'):
+                    item = RosterItem.fromElement(element)
+                    roster[item.entity] = item
+                return roster
+            else:
+                return None
 
-            return roster
-
-        iq = IQ(self.xmlstream, 'get')
-        iq.addElement((NS_ROSTER, 'query'))
-        d = iq.send()
+        request = RosterRequest(stanzaType='get')
+        request.version = version
+        d = self.request(request)
         d.addCallback(processRoster)
         return d
 
 
+    def setItem(self, item):
+        """
+        Add or modify a roster item.
+
+        Note that RFC 6121 doesn't allow all properties of a roster item to
+        be sent when setting a roster item. Only the C{name} and C{groups}
+        attributes from C{item} are sent to the server. Presence subscription
+        management must be done through L{PresenceProtocol}.
+
+        @param item: The roster item to be set.
+        @type item: L{RosterItem}.
+
+        @rtype: L{twisted.internet.defer.Deferred}
+        """
+        request = RosterRequest(stanzaType='set')
+        request.rosterSet = True
+        request.item = item
+        return self.request(request)
+
+
     def removeItem(self, entity):
         """
         Remove an item from the contact list.
 
         @param entity: The contact to remove the roster item for.
         @type entity: L{JID<twisted.words.protocols.jabber.jid.JID>}
+
         @rtype: L{twisted.internet.defer.Deferred}
         """
-        iq = IQ(self.xmlstream, 'set')
-        iq.addElement((NS_ROSTER, 'query'))
-        item = iq.query.addElement('item')
-        item['jid'] = entity.full()
-        item['subscription'] = 'remove'
-        return iq.send()
+        item = RosterItem(entity)
+        item.remove = True
+        return self.setItem(item)
 
 
     def _onRosterSet(self, iq):
-        if iq.handled or \
-           iq.hasAttribute('from') and iq['from'] != self.xmlstream:
-            return
+        def trapIgnored(failure):
+            failure.trap(RosterPushIgnored)
+            raise error.StanzaError('service-unavailable')
 
-        iq.handled = True
+        request = RosterRequest.fromElement(iq)
 
-        itemElement = iq.query.item
-
-        if unicode(itemElement['subscription']) == 'remove':
-            self.onRosterRemove(JID(itemElement['jid']))
+        if (not self.allowAnySender and
+                request.sender and
+                request.sender.userhostJID() !=
+                self.parent.jid.userhostJID()):
+            d = defer.fail(RosterPushIgnored())
+        elif request.item.remove:
+            d = defer.maybeDeferred(self.removeReceived, request)
         else:
-            item = self._parseRosterItem(iq.query.item)
-            self.onRosterSet(item)
+            d = defer.maybeDeferred(self.setReceived, request)
+        d.addErrback(trapIgnored)
+        return d
 
-    def onRosterSet(self, item):
+
+    def setReceived(self, request):
         """
         Called when a roster push for a new or update item was received.
 
-        @param item: The pushed roster item.
-        @type item: L{RosterItem}
+        @param request: The push request.
+        @type request: L{RosterRequest}
         """
+        if hasattr(self, 'onRosterSet'):
+            warnings.warn(
+                "wokkel.xmppim.RosterClientProtocol.onRosterSet "
+                "was deprecated in Wokkel 0.7.1; "
+                "please use RosterClientProtocol.setReceived instead.",
+                DeprecationWarning)
+            return defer.maybeDeferred(self.onRosterSet, request.item)
 
-    def onRosterRemove(self, entity):
+
+    def removeReceived(self, request):
         """
         Called when a roster push for the removal of an item was received.
 
-        @param entity: The entity for which the roster item has been removed.
-        @type entity: L{JID}
+        @param request: The push request.
+        @type request: L{RosterRequest}
         """
+        if hasattr(self, 'onRosterRemove'):
+            warnings.warn(
+                "wokkel.xmppim.RosterClientProtocol.onRosterRemove "
+                "was deprecated in Wokkel 0.7.1; "
+                "please use RosterClientProtocol.removeReceived instead.",
+                DeprecationWarning)
+            return defer.maybeDeferred(self.onRosterRemove,
+                                       request.item.entity)
 
 
 
